@@ -242,6 +242,7 @@ class DSLibrary(object):
         # data can be requested through a named filesystem engine/provider
         filesystem = kwargs.get("filesystem")
         if filesystem:
+            open_args.pop("filesystem", None)
             fs = self.get_filesystem_connection(filesystem, for_write="r" not in mode, **open_args)
             # TODO some of 'open_args' might be appropriate to send to open() below rather than get_filesystem_connection(), above
             return fs.open(uri, mode=mode)
@@ -671,7 +672,7 @@ class DSLibrary(object):
         mode_base = 'a' if append else 'w'
         if isinstance(content, io.IOBase):
             # copy stream content
-            with self.open_resource(resource_name, mode=mode_base + ('' if isinstance(content, io.TextIOBase) else 'b')) as f_w:
+            with self.open_resource(resource_name, mode=mode_base + ('' if isinstance(content, io.TextIOBase) else 'b'), **kwargs) as f_w:
                 while True:
                     chunk = content.read(5000000)
                     if not chunk:
@@ -684,7 +685,7 @@ class DSLibrary(object):
                     content = content.decode("utf-8")
                 mlflow.log_text(content, uri)
             else:
-                with self.open_resource(resource_name, mode=mode_base + ('' if isinstance(content, str) else 'b')) as f_w:
+                with self.open_resource(resource_name, mode=mode_base + ('' if isinstance(content, str) else 'b'), **kwargs) as f_w:
                     f_w.write(content)
         elif isinstance(content, (dict, list, tuple)):
             # write yaml or json
@@ -692,11 +693,11 @@ class DSLibrary(object):
                 mlflow.log_dict(content, uri)
                 return
             if format in ("yaml", "yml"):
-                with self.open_resource(resource_name, mode=mode_base) as f_w:
+                with self.open_resource(resource_name, mode=mode_base, **kwargs) as f_w:
                     yaml.safe_dump(content, f_w)
                     f_w.write("\n")
             else:
-                with self.open_resource(resource_name, mode=mode_base) as f_w:
+                with self.open_resource(resource_name, mode=mode_base, **kwargs) as f_w:
                     json.dump(content, f_w)
                     f_w.write("\n")
         else:
@@ -725,12 +726,12 @@ class DSLibrary(object):
                             break
                         f_w.write(chunk)
 
-    def read_resource(self, resource_name: str, mode: str='rb') -> (str, bytes):
+    def read_resource(self, resource_name: str, mode: str='rb', **kwargs) -> (str, bytes):
         """
         Completely read the contents of a given named resource.  This is a convenience method for accessing small
         files.
         """
-        with self.open_resource(resource_name, mode=mode) as f_r:
+        with self.open_resource(resource_name, mode=mode, **kwargs) as f_r:
             return f_r.read()
 
     @staticmethod
@@ -866,7 +867,7 @@ class DSLibrary(object):
                     col_names = [col[0] for col in c.description]
                     df = pandas.DataFrame(data=c, columns=col_names)
                     if use_dask:
-                        mem_size = df.memory_usage()
+                        mem_size = df.memory_usage().sum()
                         if mem_size > use_dask:
                             partition_size = max(use_dask, 40000000)
                             df = dask.dataframe.from_pandas(df, npartitions=min(mem_size//partition_size, 2))
@@ -964,35 +965,64 @@ class DSLibrary(object):
                     return read_method(uri, storage_options=storage_options, **read_args)
                 except IsADirectoryError:
                     return read_method(uri + "/*", storage_options=storage_options, **read_args)
+            # when returning a chunk stream we need to keep the stream open until iteration completes
+            if format_info.read_args.get("chunksize"):
+                class CloseAfterIteration(object):
+                    def __init__(self, stream, iterator):
+                        self.stream = stream
+                        self.iterator = iterator
+
+                    def __iter__(self):
+                        return self
+
+                    def __next__(self):
+                        try:
+                            return next(self.iterator)
+                        except StopIteration:
+                            self.stream.close()
+                            raise
+
+                    def tell(self):
+                        if not hasattr(self.stream, "tell"):
+                            return
+                        return self.stream.tell()
+                stream = use_opener(uri, mode='rb', **open_args)
+                iterator = self._open_from_stream(stream, read_method, read_args)
+                return CloseAfterIteration(stream, iterator)
             with use_opener(uri, mode='rb', **open_args) as f_r:
-                if format == "xlsx" and not f_r.seekable():
-                    # the XLSX reader requires its stream to be seekable so we download to a temporary file
-                    with tempfile.NamedTemporaryFile(mode='wb') as f_tmp:
-                        # chunked copy
-                        shutil.copyfileobj(f_r, f_tmp)
-                        f_tmp.flush()
-                        with open(f_tmp.name, mode='rb') as f_r2:
-                            return read_method(f_r2, **read_args)
-                # the 'hdf' implementation doesn't accept a stream, only a filename
-                if format == "hdf" and hasattr(f_r, "name") and ":" not in f_r.name:
-                    return read_method(f_r.name, **read_args)
-                if format == "hdf":
-                    with tempfile.NamedTemporaryFile(mode='wb') as f_tmp:
-                        shutil.copyfileobj(f_r, f_tmp)
-                        f_tmp.flush()
-                        return read_method(f_tmp.name, **read_args)
-                try:
-                    return read_method(f_r, **read_args)
-                except UnicodeDecodeError as err:
-                    raise DSLibraryDataFormatException(f"Invalid character encoding: {err.reason}: encoding={err.encoding}, offset={err.start} - set 'encoding' or 'encoding_errors' options to compensate", offset=err.start)
-                except ValueError as err:
-                    if format == "json":
-                        raise DSLibraryDataFormatException(f"JSON format error: {str(err)}")
-                    else:
-                        raise DSLibraryDataFormatException(f"{str(err)}")
+                return self._open_from_stream(f_r, read_method, read_args)
         # TODO support read from folder, i.e. folder of CSVs, parquet, etc.
         raise DSLibraryException(f"file type not supported: {format}")
 
+    def _open_from_stream(self, f_r, read_method, read_args):
+        if format == "xlsx" and not f_r.seekable():
+            # the XLSX reader requires its stream to be seekable so we download to a temporary file
+            with tempfile.NamedTemporaryFile(mode='wb') as f_tmp:
+                # chunked copy
+                shutil.copyfileobj(f_r, f_tmp)
+                f_tmp.flush()
+                with open(f_tmp.name, mode='rb') as f_r2:
+                    return read_method(f_r2, **read_args)
+        # the 'hdf' implementation doesn't accept a stream, only a filename
+        if format == "hdf" and hasattr(f_r, "name") and ":" not in f_r.name:
+            return read_method(f_r.name, **read_args)
+        if format == "hdf":
+            with tempfile.NamedTemporaryFile(mode='wb') as f_tmp:
+                shutil.copyfileobj(f_r, f_tmp)
+                f_tmp.flush()
+                return read_method(f_tmp.name, **read_args)
+        try:
+            return read_method(f_r, **read_args)
+        except UnicodeDecodeError as err:
+            raise DSLibraryDataFormatException(
+                f"Invalid character encoding: {err.reason}: encoding={err.encoding}, offset={err.start} - set 'encoding' or 'encoding_errors' options to compensate",
+                offset=err.start)
+        except ValueError as err:
+            if format == "json":
+                raise DSLibraryDataFormatException(f"JSON format error: {str(err)}")
+            else:
+                raise DSLibraryDataFormatException(f"{str(err)}")
+
     def _default_format_for_aliases(self, resource_name: str, options: dict):
         """
         Fill in default format for the built-in aliases.
